import 'dart:developer';

import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:webview_flutter/webview_flutter.dart';

import '../cubit/saml_authentication_cubit.dart';

class SAMLAuthenticationWebview extends StatefulWidget {
  final String tenantDomain;
  const SAMLAuthenticationWebview({
    Key? key,
    required this.tenantDomain,
  }) : super(key: key);

  @override
  State<SAMLAuthenticationWebview> createState() =>
      _SAMLAuthenticationWebviewState();
}

class _SAMLAuthenticationWebviewState extends State<SAMLAuthenticationWebview> {
  late final WebViewController _webViewController;

  late final WebViewCookieManager _cookieManager;

  @override
  void initState() {
    super.initState();

    _cookieManager = WebViewCookieManager();
    _cookieManager.clearCookies();

    _webViewController = WebViewController()
      ..setJavaScriptMode(JavaScriptMode.unrestricted)
      ..setBackgroundColor(const Color(0x00000000))
      ..clearCache()
      ..clearLocalStorage()
      ..setNavigationDelegate(
        NavigationDelegate(
          onPageStarted: (String url) {
            log('>>>>> PAGE STARTED ==> $url');
          },
          onPageFinished: (String url) {
            log('>>>>> PAGE FINISHED ==> $url');
            _checkForSeniorXCookies();
          },
          onWebResourceError: (WebResourceError error) {},
          onNavigationRequest: (NavigationRequest request) {
            // Blocking any navigation to prevent Senior X loads
            return request.url
                    .startsWith('https://platform.senior.com.br/senior-x/#/')
                ? NavigationDecision.prevent
                : NavigationDecision.navigate;
          },
        ),
      )
      ..loadRequest(Uri.parse('https://platform.senior.com.br/login/'
          '?redirectTo=https://platform.senior.com.br'
          '&tenant=${widget.tenantDomain}'
          '&check-browser-compatibility=false'));
  }

  @override
  Widget build(BuildContext context) {
    return WebViewWidget(
      controller: _webViewController,
    );
  }

  void _checkForSeniorXCookies() async {
    final cookies = await _webViewController
        .runJavaScriptReturningResult('document.cookie');

    if (context.mounted) {
      context.read<SAMLAuthenticationCubit>().verifySeniorXCookies(cookies);
    }
  }
}
