import 'package:flutter_test/flutter_test.dart';
import 'package:mocktail/mocktail.dart';
import 'package:senior_platform_authentication/senior_platform_authentication.dart';
import 'package:senior_platform_authentication_ui/src/core/authentication/data/repositories/secure_storage_repository.dart';
import 'package:senior_platform_authentication_ui/src/core/authentication/domain/usecases/clear_stored_data_usecase.dart';

class MockSecureStorageRepository extends Mock
    implements SecureStorageRepository {}

void main() {
  late final ClearStoredDataUsecase clearStoredDataUsecase;
  late final SecureStorageRepository repository;

  setUpAll(() {
    repository = MockSecureStorageRepository();
    clearStoredDataUsecase =
        ClearStoredDataUsecase(secureStorageRepository: repository);
  });

  group('clearStoredDataUsecase', () {
    test('should execute successfully', () async {
      // Arrange
      when(
        () => repository.deleteUser(),
      ).thenAnswer((_) async {
        return;
      });

      when(
        () => repository.deleteToken(),
      ).thenAnswer((_) async {
        return;
      });

      when(
        () => repository.deleteTokenExpirationDate(),
      ).thenAnswer((_) async {
        return;
      });

      verifyNever(
        () => repository.deleteUser(),
      );

      verifyNever(
        () => repository.deleteToken(),
      );

      verifyNever(
        () => repository.deleteTokenExpirationDate(),
      );

      // Act
      await clearStoredDataUsecase(NoParams());

      // Assert
      verify(
        () => repository.deleteUser(),
      ).called(1);

      verify(
        () => repository.deleteToken(),
      ).called(1);

      verify(
        () => repository.deleteTokenExpirationDate(),
      ).called(1);
    });

    test('should throw exception when repository throws', () async {
      // Arrange
      when(
        () => repository.deleteUser(),
      ).thenThrow(Exception('oops'));

      when(
        () => repository.deleteToken(),
      ).thenAnswer((_) async {
        return;
      });

      when(
        () => repository.deleteTokenExpirationDate(),
      ).thenAnswer((_) async {
        return;
      });

      verifyNever(
        () => repository.deleteUser(),
      );

      verifyNever(
        () => repository.deleteToken(),
      );

      verifyNever(
        () => repository.deleteTokenExpirationDate(),
      );

      // Act
      final action = clearStoredDataUsecase(NoParams());

      // Assert
      expect(
        () async => await action,
        throwsA(isA<Exception>()),
      );

      // Assert
      verify(
        () => repository.deleteUser(),
      ).called(1);

      verifyNever(
        () => repository.deleteToken(),
      );

      verifyNever(
        () => repository.deleteTokenExpirationDate(),
      );
    });
  });
}
